/*********
  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp32-cam-take-photo-display-web-server/
  
  IMPORTANT!!! 
   - Select Board "AI Thinker ESP32-CAM"
   - GPIO 0 must be connected to GND to upload a sketch
   - After connecting GPIO 0 to GND, press the ESP32-CAM on-board RESET button to put your board in flashing mode
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
*********/

#include "WiFi.h"
#include "esp_camera.h"
#include "esp_timer.h"
#include "img_converters.h"
#include "Arduino.h"
#include "soc/soc.h"           // Disable brownour problems
#include "soc/rtc_cntl_reg.h"  // Disable brownour problems
#include "driver/rtc_io.h"
#include <ESPAsyncWebServer.h>
#include <StringArray.h>
#include <SPIFFS.h>
#include <FS.h>

// button, led
const int pinButton = 2;
const int pinLed = 13;
int lastButton=1;          // последнее состояние кнопки
int currentButton=1;       // текущее состояние кнопки
unsigned long millisbutton=0;  // длительность нажатия мсек

// static ip
// Задайте свой статический IP-адрес
IPAddress local_IP(192, 168, 1, 61);
// Укажите IP-адрес шлюза
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0);
IPAddress primaryDNS(192, 168, 1, 1);   // опционально
IPAddress secondaryDNS(8, 8, 8, 8); // опционально
// Replace with your network credentials
const char* ssid = "Barbie_city";
const char* password = "U4TT6KGU";

// Create AsyncWebServer object on port 80
AsyncWebServer server(80);

boolean takeNewPhoto = false;
boolean printPhoto = false;

String serverName = "192.168.1.60";   // 
String serverPath = "/photo_barbie/upload.php";     // The default serverPath should be upload.php
const int serverPort = 80;
WiFiClient client;

// Photo File Name to save in SPIFFS
#define FILE_PHOTO "/photo.jpg"

// OV2640 camera module pins (CAMERA_MODEL_AI_THINKER)
#define PWDN_GPIO_NUM     32
#define RESET_GPIO_NUM    -1
#define XCLK_GPIO_NUM      0
#define SIOD_GPIO_NUM     26
#define SIOC_GPIO_NUM     27
#define Y9_GPIO_NUM       35
#define Y8_GPIO_NUM       34
#define Y7_GPIO_NUM       39
#define Y6_GPIO_NUM       36
#define Y5_GPIO_NUM       21
#define Y4_GPIO_NUM       19
#define Y3_GPIO_NUM       18
#define Y2_GPIO_NUM        5
#define VSYNC_GPIO_NUM    25
#define HREF_GPIO_NUM     23
#define PCLK_GPIO_NUM     22

const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html>
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <style>
    body { text-align:center; }
    .vert { margin-bottom: 10%; }
    .hori{ margin-bottom: 0%; }
  </style>
</head>
<body>
  <div>
  <div id="container"><center>
    <h2>BARBIE фотосалон</h2>
    <p>
      <button onclick="capturePhoto()">ПРОСМОТР</button><br>
      <button onclick="printPhoto();">ПЕЧАТЬ</button>
      <br><br><br><br><br><br>
    </p></center>
  </div>
  <div><center><img src="saved-photo" id="photo" style="transform: rotate(270deg);" width="480px"></center></div>
  </div>
</body>
<script>
  var deg = 180;
  function printPhoto() {
    var img = document.getElementById("photo");
    img.src="http:/"+"/192.168.1.60/photo_barbie/printer.gif";
    img.style.transform ="rotate(0deg)";
    var xhr = new XMLHttpRequest();
    xhr.open('GET', "/print", true);
    xhr.onload = function() {
      if (xhr.readyState == 4 && xhr.status == 200) {
        //alert("printing");
        setTimeout((function() {window.location.reload();}), 5000);
      }
    }
    xhr.send();    
  }
  function capturePhoto() {
    var img = document.getElementById("photo");
    img.src="http:/"+"/192.168.1.60/photo_barbie/load.gif";
    var xhr = new XMLHttpRequest();
    xhr.open('GET', "/capture", true);
    xhr.onload = function() {
      if (xhr.readyState == 4 && xhr.status == 200) {
        setTimeout((function() {window.location.reload();}), 5000);
      }
    }
    xhr.send();    
  }
  function rotatePhoto() {
    var img = document.getElementById("photo");
    deg += 90;
    if(isOdd(deg/90)){ document.getElementById("container").className = "vert"; }
    else{ document.getElementById("container").className = "hori"; }
    img.style.transform = "rotate(" + deg + "deg)";
  }
  function isOdd(n) { return Math.abs(n % 2) == 1; }
</script>
</html>)rawliteral";



void setup() {
  // Serial port for debugging purposes
  Serial.begin(115200);
  //
  pinMode(pinButton, INPUT_PULLUP);
  pinMode(pinLed, OUTPUT);
  digitalWrite(pinLed,HIGH);
  
  // Настраивает статический IP-адрес
  if (!WiFi.config(local_IP, gateway, subnet, primaryDNS, secondaryDNS)) {
    Serial.println("STA Failed to configure");
  }
  // Connect to Wi-Fi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi...");
  }
  if (!SPIFFS.begin(true)) {
    Serial.println("An Error has occurred while mounting SPIFFS");
    ESP.restart();
  }
  else {
    delay(500);
    Serial.println("SPIFFS mounted successfully");
  }

  // Print ESP32 Local IP Address
  Serial.print("IP Address: http://");
  Serial.println(WiFi.localIP());

  // Turn-off the 'brownout detector'
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 0);

  // OV2640 camera module
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sccb_sda = SIOD_GPIO_NUM;
  config.pin_sccb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;

  if (psramFound()) {
    config.frame_size = FRAMESIZE_VGA;
    config.jpeg_quality = 14;
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_VGA;
    config.jpeg_quality = 14;
    config.fb_count = 1;
  }
  // Camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed with error 0x%x", err);
    ESP.restart();
  }

  // Route for root / web page
  server.on("/", HTTP_GET, [](AsyncWebServerRequest * request) {
    request->send_P(200, "text/html", index_html);
  });

  server.on("/capture", HTTP_GET, [](AsyncWebServerRequest * request) {
    takeNewPhoto = true;
    request->send_P(200, "text/plain", "Taking Photo");
  });

  server.on("/print", HTTP_GET, [](AsyncWebServerRequest * request) {
    printPhoto = true;
    request->send_P(200, "text/plain", "Printing Photo");
  });

  server.on("/saved-photo", HTTP_GET, [](AsyncWebServerRequest * request) {
    request->send(SPIFFS, FILE_PHOTO, "image/jpg", true);
  });

  // Start server
  server.begin();
  //
  digitalWrite(pinLed,LOW);
  Serial.println("start");

}

void loop() {
  if (takeNewPhoto) {
    capturePhotoSaveSpiffs();
    takeNewPhoto = false;
  }
  if (printPhoto) {
    printPhotoSite();
    printPhoto = false;
  }
  delay(1);
  // ********  сканирование кнопки
  currentButton = debounce(lastButton,pinButton);
  // если нажатие...
  if (lastButton == 1 && currentButton == 0) {
      Serial.println("click");
      millisbutton=millis();
  }
  //если отжатие
  else if (lastButton == 0 && currentButton == 1) {
      //Serial.println(millisbutton-millis());
      if(millis()-millisbutton >= 2000) {
        Serial.println("long onclick");
        printPhotoButton();
      }
      else {
        Serial.println("onclick");
      }
   }   
   lastButton  = currentButton;   
  /////////////
}
